
/* nxlcau5t.c				*/
/* (5,3/2) Linear Cellular Automaton	*/

/* Reference:				*/
/*					*/
/*    Kenneth E. Perry			*/
/*    Abstract Mathematical Art		*/
/*    BYTE				*/
/*    December, 1986			*/
/*    pages 181-192			*/

/*    Copyright (C) 1987		*/
/*    Copyright (C) 1988		*/
/*    Copyright (C) 1989		*/
/*    Copyright (C) 1990		*/
/*    Copyright (C) 1994		*/
/*    Copyright (C) 1995		*/
/*    Harold V. McIntosh		*/
/*    Gerardo Cisneros S.		*/

/* G. Cisneros, 4.3.87						*/
/* 28 April 1987 - version for XVI Feria de Puebla [HVM]	*/
/* 14 June 1987 - individual cycles of evolution [HVM]		*/
/* 17 June 1987 - p-adic representation in plane [HVM]		*/
/* 22 June 1987 - 2 speed gliders via 16 transitions [HVM]	*/
/* 26 June 1987 - push, pop the rule [HVM]			*/
/* 26 June 1987 - conserve position of rule cursor [HVM]	*/
/* 27 June 1987 - incremental rule construction [HVM]		*/
/* 29 June 1987 - conserve position of cell pointer [HVM]	*/
/* 30 June 1987 - mark & unmark transitions, xchg x&X [HVM]	*/
/* 25 July 1987 - display and edit de Bruijn diagrams [HVM]	*/
/* 27 July 1987 - graph of transition probabilities [HVM]	*/
/* 4 September 1987 - PROB41.C for option 't' [HVM]		*/
/* 21 October 1987 - program disks disappeared			*/
/* 24 December 1987 - program reconstructed from listings	*/
/* 20 February 1988 - RIJN31.C for option 'd' [HVM]		*/
/* 10 April 1988 - include rule engineering from LCA41 [HVM]	*/
/* 11 March 1989 - option 'x' in trule [HVM]			*/
/* 17 September 1989 - directory for demonstration rules [HVM]	*/
/* 5 October 1989 - modified for radius 1/2 [HVM]		*/
/* 10 February 1990 - Garden of Eden option [HVM]		*/

// # include <bdos.h>
# include "nxlcau5t.h"
# include "rec.h"

# define LROW		 9	/* top row of line panel  */
# define DROWS		20	/* rows in demo menu	  */
# define DCOLS		32	/* second col in demo m.  */

# define NX          	14	/* number of sample rules */

char ixrule[NX][TS]= {

	"000101011",	/*  1 */
	"000101101",	/*  2 */
	"000101101",	/*  3 */
	"000101101",	/*  4 */
	"000101101",	/*  5 */
	"000101111",	/*  6 */
	"002011220",	/*  7 gridwork */
	"000110111",	/*  8 */
	"001011111",	/*  9 */
	"122201202",	/* 10 class iv */
	"010110101",	/* 11 */
	"011010011",	/* 12 */
	"200011202",	/* 13 colored snow */
	"110100101"	/* 14 */

	};

parneih    amat;
parneih    bmat;
parneih    cmat;
parneih    dmat;
parneih    emat;
parneih    fmat;
parneih    gmat;
parneih    hmat;
parneih    xmat;

// twoparn    smat;
// twoparn    tmat;
// twoparn    umat;
// twoparn    vmat;

double stt, sta[5][K3+1];
double blpr[KK];			/* probability from blocks	*/

Inst 	xprg[10][XLEN];			
Inst   	rprg[16384];			/* rec program array */
char   	cstr[CLEN];			/* console rec program */
char   	cprg[CLEN];			/* edit stored program */
char	idef [1][30] = {"(z)!100!0r:;)1r2r(0r:;);)a"};

extern int rec_c(char *z, Inst rprg, int l, struct fptbl dtbl[]);

char   tabl[20][SL];			/* workspace to construct table */
char   ascrule[KK][KK][KK][KK];		/* ASCII transition values */
char   auxrule[KK][KK][KK][KK];		/* auxiliary transition table */
char   rulstk[10][KK][KK][KK][KK];	/* pushdown for rules */
char   prule1[8], prule2[8];		/* product rule specification */
char   strule[KK+1][SS-1];
char   gf3rule[13];			/* coefficients of GF3 rule	*/
char   ttrule[TS];
char   tarule[TS];
char   srule[KK+1][TS-2];
char   ascrule[KK][KK][KK][KK];
char   auxrule[KK][KK][KK][KK];		/* auxiliary transition table */
int    binrule[KK][KK][KK][KK];
int    co, dn;					/* connectivity, dimension */

char   arule[DS], arr1[AL], arr2[AL], arr3[AL];
int    ru, ru0, ru1, ru2, ru3;			/* rule cursor */
int    li, lj;					/* cell pointer */
int    di, dj;					/* demo pointer */
int    sc;					/* screen colors */
int    rulptr;					/* rule pd ptr */
int    ix0, iy0;				/* origin for pen moves */
double wmul[KK], wmvl[KK];			/* left mass point */
double wmur[KK], wmvr[KK];			/* right mass point */


// # include "nxlcaukr.c"


lcau5tmain() {}    

/* list of demonstration rules */
int dmenu() {}

/* edit the rule */
edrule() {char c;
  while (0<1) {}
}

/* advance the rule cursor */
rupl() {
  ru3++;
  if (ru3==KK) {ru3=0; ru2++;}
  if (ru2==KK) {ru2=0; ru1++;}
  if (ru1==KK) {ru1=0; ru0++;}
  if (ru0==KK) {ru0=0; }
  ru=KK*(KK*(KK*ru0+ru1)+ru2)+ru3+6;
  }

/* retract the rule cursor */
rumi() {
  if (ru3!=0) ru3--; else {ru3=KK-1;
  if (ru2!=0) ru2--; else {ru2=KK-1;
  if (ru1!=0) ru1--; else {ru1=KK-1;
  if (ru0!=0) ru0--; else {ru0=KK-1;
     }}}}
  ru=KK*(KK*(KK*ru0+ru1)+ru2)+ru3+6;
  }

/* edit totalistic rule */
edtrule() {char c; int  i, k;
  i=0;
  while (i<TS) {}
}

/* edit the line of cells */
edline(nr,nc) int nr, nc; {}       /* end edline */

/* calculate index, or rather the neighbors, in edline. */
linix(xx,i,j,nc) int xx[RR+1], i, j, nc; {int k, x;
  xx[(RR-1)/2]=x=i+nc*(j-1);
  if (i==0) xx[0]=x+nc-1; else xx[0]=x-1;
  if (i==nc-1) xx[2]=x-nc+1; else xx[2]=x+1;
  for (k=0; k<RR; k++) xx[k]=arr1[xx[k]];
  xx[RR]=arr1[x+nc];
}

/* display a screen of evolution */
evolve() {int i, j;
  asctobin();
  for (j=0; j<200; j++) {		/* evolve for 200 generations */
    for (i=0; i<YL; i++) videodot(j,i,arr1[i]);
    onegen(YL);
    }
}

/* copy ascrule over to binrul */
asctobin() {int i0, i1, i2, i3;
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
      binrule[i0][i1][i2][i3]=ascrule[i0][i1][i2][i3]-'0';
}

/* evolution for one generation */
/* j = number of cells in array */

onegen(j) int j; {int i, k, i0, i1, i2, i3; long int ii;
switch (dn) {
case 1:
if (j>3) {
  arr2[0]=binrule[arr1[j-2]][arr1[j-1]][arr1[0]][arr1[1]];
  arr2[1]=binrule[arr1[j-1]][arr1[0]][arr1[1]][arr1[2]];
  for (i=2; i<j-1; i++) 
    arr2[i]=binrule[arr1[i-2]][arr1[i-1]][arr1[i]][arr1[i+1]];
  arr2[j-2]=binrule[arr1[j-4]][arr1[j-3]][arr1[j-2]][arr1[j-1]];
  arr2[j-1]=binrule[arr1[j-3]][arr1[j-2]][arr1[j-1]][arr1[0]];
} else if (j==3) {
  arr2[0]=binrule[arr1[1]][arr1[2]][arr1[0]][arr1[1]]; 
  arr2[1]=binrule[arr1[2]][arr1[0]][arr1[1]][arr1[2]]; 
  arr2[2]=binrule[arr1[0]][arr1[1]][arr1[2]][arr1[0]]; 
} else if (j==2) {
  arr2[0]=binrule[arr1[0]][arr1[1]][arr1[0]][arr1[1]]; 
  arr2[1]=binrule[arr1[1]][arr1[0]][arr1[1]][arr1[0]]; 
} else if (j==1) {
  arr2[0]=binrule[arr1[0]][arr1[0]][arr1[0]][arr1[0]]; 
}
if (co==1) for (ii=0; ii<j; ii++) arr1[(int)ii]=arr2[(int)(ii)];
if (co==0) for (ii=0; ii<j; ii++) arr1[(int)ii]=arr2[(int)((7*ii)%j)];
if (co==2) for (ii=0; ii<j; ii+=2) {arr1[(int)ii]=arr2[(int)((7*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((7*ii+1)%j)];}
if (co==3) for (ii=0; ii<j; ii+=3) {arr1[(int)ii]=arr2[(int)((7*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((7*ii+1)%j)];
				 arr1[(int)ii+2]=arr2[(int)((7*ii+2)%j)];}
if (co==4) for (ii=0; ii<j; ii+=4) {arr1[(int)ii]=arr2[(int)((7*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((7*ii+1)%j)];
				 arr1[(int)ii+2]=arr2[(int)((7*ii+2)%j)];
				 arr1[(int)ii+3]=arr2[(int)((7*ii+3)%j)];}

if (co==5) {for (ii=0; ii<j; ii+=2) {arr1[(int)ii]=arr2[(int)((11*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((11*ii+1)%j)];}
	    for (ii=0; ii<j; ii+=8) {
		k=arr1[(int)ii];
		arr1[(int)ii]=arr1[(int)((ii+2)%j)];
		arr1[(int)((ii+2)%j)]=k;
		k=arr1[(int)((ii+1)%j)];
		arr1[(int)((ii+1)%j)]=arr1[(int)((ii+4)%j)];
		arr1[(int)((ii+4)%j)]=k;
		k=arr1[(int)((ii+3)%j)];
		arr1[(int)((ii+3)%j)]=arr1[(int)((ii+5)%j)];
		arr1[(int)((ii+5)%j)]=k;
                }
	    }
if (co==6) {for (ii=0; ii<j; ii+=2) {arr1[(int)ii]=arr2[(int)((11*ii)%j)];
				 arr1[(int)ii+1]=arr2[(int)((11*ii+1)%j)];}
	    for (ii=0; ii<j; ii+=16) {
		k=arr1[(int)ii];
		arr1[(int)ii]=arr1[(int)((ii+2)%j)];
		arr1[(int)((ii+2)%j)]=k;
		k=arr1[(int)((ii+1)%j)];
		arr1[(int)((ii+1)%j)]=arr1[(int)((ii+4)%j)];
		arr1[(int)((ii+4)%j)]=k;
		k=arr1[(int)((ii+3)%j)];
		arr1[(int)((ii+3)%j)]=arr1[(int)((ii+5)%j)];
		arr1[(int)((ii+5)%j)]=k;

		k=arr1[(int)((ii+8)%j)];
		arr1[(int)((ii+8)%j)]=arr1[(int)((ii+10)%j)];
		arr1[(int)((ii+10)%j)]=k;
		k=arr1[(int)((ii+9)%j)];
		arr1[(int)((ii+9)%j)]=arr1[(int)((ii+15)%j)];
		arr1[(int)((ii+15)%j)]=k;
		k=arr1[(int)((ii+11)%j)];
		arr1[(int)((ii+11)%j)]=arr1[(int)((ii+13)%j)];
		arr1[(int)((ii+13)%j)]=k;
		k=arr1[(int)((ii+12)%j)];
		arr1[(int)((ii+12)%j)]=arr1[(int)((ii+14)%j)];
		arr1[(int)((ii+14)%j)]=k;
		}
	    } break;

/* Moore (3,1/2) in two dimensions */

case 2: if (j<16384) break;
	  for (i0=0; i0<128; i0++) for (i1=0; i1<128; i1++) 
	    arr2[ix2(i0,i1)]=binrule
	      [arr1[ix2(i0,i1)]][arr1[ix2(i0,i1+1)]]
	      [arr1[ix2(i0+1,i1)]][arr1[ix2(i0+1,i1+1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* the corner cell on a cube */

case 3: if (j<16384) break;
	  for (i0=0; i0<32; i0++)
	  for (i1=0; i1<32; i1++)
	  for (i2=0; i2<16; i2++)
	    arr2[ix3(i0,i1,i2)]=binrule
	      [arr1[ix3(i0+1,i1,i2)]][arr1[ix3(i0,i1+1,i2)]]
	      [arr1[ix3(i0,i1,i2+1)]][arr1[ix3(i0,i1,i2)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* the corner cell on a hypercube */

case 4: if (j<16384) break;
	  for (i0=0; i0<16; i0++) for (i1=0; i1<8; i1++) 
	  for (i2=0; i2<16; i2++) for (i3=0; i3<8; i3++) 
	    arr2[ix4(i0,i1,i2,i3)]=binrule
	      [arr1[ix4(i0+1,i1,i2,i3)]][arr1[ix4(i0,i1+1,i2,i3)]]
	      [arr1[ix4(i0,i1,i2+1,i3)]][arr1[ix4(i0,i1,i2,i3+1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* von Neumann (3,1) in two dimensions without the center cell */

case 5: if (j<16384) break;
	  for (i0=0; i0<128; i0++) for (i1=0; i1<128; i1++) 
	    arr2[ix2(i0,i1)]=binrule
	      [arr1[ix2(i0,i1-1)]][arr1[ix2(i0,i1+1)]]
	      [arr1[ix2(i0-1,i1)]][arr1[ix2(i0+1,i1)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

/* the center cell of a tetrahedron */

case 6: if (j<16384) break;
	  for (i0=0; i0<32; i0++)
	  for (i1=0; i1<32; i1++)
	  for (i2=0; i2<16; i2++)
	    arr2[ix3(i0,i1,i2)]=binrule
	      [arr1[ix3(i0,i1+1,i2-1)]][arr1[ix3(i0,i1+1,i2+1)]]
	      [arr1[ix3(i0+1,i1-1,i2)]][arr1[ix3(i0-1,i1-1,i2)]];
	  for (i=0; i<16384; i++) arr1[i]=arr2[i];
	    break;

default: break;}
}

/* index two dimensions */
int ix2(i,j) int i, j; {return (128+i)%128+128*((128+j)%128);}

/* index three dimensions */
int ix3(i,j,k) int i, j, k; 
{return (32+i)%32+ 32*((32+j)%32+ 32*((16+k)%16 ));}

/* index four dimensions */
int ix4(i,j,k,l) int i, j, k, l; {
return (16+i)%16+ 16*( (8+j)%8+ 8*( (16+k)%16+ 16*( (8+l)%8 )));}

/* uniform line menu */
ulmenu(l) int l; {int i, j, k;
  for (j=0; j*SL<AL; j++) {
  for (i=0; i<SL;    i++) {
    videocursor(0,LROW+j,i);
    k=arr1[j*SL+i];
    videocattr(0,k==0?'.':k+'0',l,1);
    }}
  }

/* map the automaton state into the unit square */
pprob() {int i, j; float x, y, z, k;
i=AL/2; k=(float)KK; z=(float)((197/(KK*KK))*KK*KK);
x=0.0; for (j=0; j<30; j++) x=(x+(float)arr1[i+30-j])/k;
y=0.0; for (j=0; j<30; j++) y=(y+(float)arr1[i-29+j])/k;
videodot((int)(z*(1.0-y)),(int)(z*x),1);
}

/* set up a grid for the unit square */
pgrid() {int i, j, k, l;
k=KK*KK+1;
l=195/(k-1);
for (i=0; i<k; i++) {
for (j=0; j<k; j++) {
videodot(l*i,l*j,2);
}}
}

/* tutorial and Help screen */
tuto() {}

/* rule menu */
rmenu() {}

/* totalistic rule menu*/
tmenu() {}

/* plain line menu */
lmenu() {int i, j; char c;
  for (j=0; SL*j<AL; j++) {videocursor(0,LROW+j,0);
  for (i=0; i<SL; i++) {c='0'+arr1[SL*j+i]; videoputc(c=='0'?'.':c,1); } }
  videocursor(0,5,0); }

/* color line menu */
clmenu() {int c, i, j;
  for (j=0; SL*j<AL; j++) {
    for (i=0; i<SL; i++) {
      videocursor(0,LROW+j,i);
      c=arr1[SL*j+i];
      videocattr(0,c==0?'.':'0'+c,color(c),1);
      }}
  videocursor(0,5,0); }

/* get compatible color */
int color(i) int i; {
  switch (i) {
    case 0:  return 2;
    case 1:  return 3;
    case 2:  return 5;
    case 3:  return 7;
    default: return 6;
}}

/* display Wolfram rule number */
wmenu(n) int n;
{char xx[6]; int i, nn;  
    nn=sprintf(xx,"%5d",n);
    videocursor(0,2,40);
    for (i=0; i<nn; i++) videoputc(xx[i],1);
    videocursor(0,2,40); }

/* display rule number */
xmenu(n) int n; {
char xx[6]; int i, nn;  
  nn=sprintf(xx,"%5u",n);
  videocursor(0,XRR,XRC);
  for (i=0; i<nn; i++) videoputc(xx[i],1);
  videocursor(0,XRR,XRC); }

/* clear screen space for rule number */
xblnk() {}

/* copy saved rule #n into active rule */
xtoasc(n) int n; {int i0, i1, i2, i3;
//  xmenu(n+1);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    ascrule[i0][i1][i2][i3]=ixrule[n][((i0*KK+i1)*KK+i2)*KK+i3];
  }

/* change totalistic rule to general rule */
totoasc() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]=ttrule[i0+i1+i2+i3];
}

/* change decimal totalistic rule to sum values */
nutoto(x) int x; {int i;
for (i=0; i<TS; i++) {ttrule[i]=x%KK+'0'; x/=KK;}}

/* change sum values to decimal totalistic rule */
int totonu(i) int i; {int r;
  if (i<TS) r=(ttrule[i]-'0')+KK*totonu(i+1); else r=0;
  return r; }

/* test whether a rule is totalistic */
int istot() {char totrule[TS]; int i0, i1, i2, i3, l;
    l=1;
    totrule[0]=ascrule[0][0][0][0];
    totrule[1]=ascrule[1][0][0][0];
    totrule[2]=ascrule[2][0][0][0];
    totrule[3]=ascrule[3][0][0][0];
    totrule[4]=ascrule[4][0][0][0];
    totrule[5]=ascrule[4][1][0][0];
    totrule[6]=ascrule[4][2][0][0];
    totrule[7]=ascrule[4][3][0][0];
    totrule[8]=ascrule[4][4][0][0];
    totrule[9]=ascrule[4][4][1][0];
    totrule[10]=ascrule[4][4][2][0];
    totrule[11]=ascrule[4][4][3][0];
    totrule[12]=ascrule[4][4][4][0];
    totrule[13]=ascrule[4][4][4][1];
    totrule[14]=ascrule[4][4][4][2];
    totrule[15]=ascrule[4][4][4][3];
    totrule[16]=ascrule[4][4][4][4];
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
      if (ascrule[i0][i1][i2][i3]!=totrule[i0+i1+i2+i3]) l=0;
    return l; }

/* find the totalistic rule number of a known ascii totalistic rule */
void asctotot() {char trule[TS]; int i0, i1, i2, i3;
    for (i0=0; i0<KK; i0++)
    for (i1=0; i1<KK; i1++)
    for (i2=0; i2<KK; i2++)
    for (i3=0; i3<KK; i3++)
      ttrule[i0+i1+i2+i3] = ascrule[i0][i1][i2][i3];
}

/* generate a random rule */
ranrul() {int i, i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  i=(random()&0xFF)%KK;
  ascrule[i0][i1][i2][i3]=i+'0';
  }
}

/* generate a random protected rule */
ranprul() {int i0, i1, i2, i3, i, j;
i=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  if (i%8 == 0) j=rand(); i++;
  if (auxrule[i0][i1][i2][i3]==' ') ascrule[i0][i1][i2][i3]='0'+j%KK;
  j/=KK;
  }
}

/* generate a random totalistic rule */
rantrul() {int i0, i, j;
i=0;
for (i0=0; i0<TS; i0++) {
  if (i%8 == 0) j=rand(); i++;
  ttrule[i0]=j%KK+'0'; j/=KK;
  }
}

/* generate a random line of cells in arr1 */
ranlin() {int i, c;
for (i=0; i<AL; i++) {
  if (i%8 == 0) c=random();
  arr1[i]=c%KK; c/=KK;}
}

/* generate a random core for a planar array */
ranplacor() {int i, j;
  for (i=60; i<69; i++) for (j=60; j<69; j++)  arr1[ix2(i,j)] = random()%KK;}

/* generate a random core for a solid array */
ransolcor() {int i, j, k;
  for (i=14; i<19; i++) for (j=14; j<19; j++) for (k=7; k<10; k++) 
	 arr1[ix3(i,j,k)] = random()%KK;}

/* clear auxrule to blanks */
auxblnk() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  auxrule[i0][i1][i2][i3]=' ';
}

/* clear ascrule to zeroes */
asczero() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]='0';
}

/* clear auxrule to zeroes */
auxzero() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  auxrule[i0][i1][i2][i3]=0;
}

/* change semitotalistic rule to general rule */
sttoasc() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  ascrule[i0][i1][i2][i3]=strule[i1+i2][i0+i3];
}

/* clear ttrule to zeroes */
ttzero() {int i; for (i=0; i<TS; i++)  ttrule[i]='0';}

/* clear strule to zeroes */
stzero() {int i, j;
  for (i=0; i<KK+1; i++) for (j=0; j<SS-1; j++)  strule[i][j]='0';
}

/* clear gf3rule to zeroes */
gfzero() {int i; for (i=0; i<13; i++) gf3rule[i]='0';}


/* generate a random line of cells in arr1 with assigned probabilities */
ranblin(p1,p2,p3,p4,p5) double p1, p2, p3, p4, p5; {
int    i, c, i1, i2, i3, i4, i5;
double p;
  p=p1+p2+p3+p4+p5;
  i1=   (int)((1000.0*p1)/p);
  i2=i1+(int)((1000.0*p2)/p);
  i3=i2+(int)((1000.0*p3)/p);
  i4=i3+(int)((1000.0*p4)/p);
  i5=i4+(int)((1000.0*p5)/p);
  for (i=0; i<AL; i++) {
    c=random()%1000;
    if      (c<i1) arr1[i]=0;
    else if (c<i2) arr1[i]=1;
    else if (c<i3) arr1[i]=2;
    else if (c<i4) arr1[i]=3;
    else           arr1[i]=4;
    }
}

/* put a single i-cell on a line of j-cells */
void putionj(i,j) int i, j; 
{int k; for (k=0; k<AL; k++) arr1[k]=j; arr1[AL/2]=i;}

/* make all of arr1 equal to zero */
zerline() {int i; for (i=0; i<AL; i++) arr1[i]=0;}

/* limit j to interval (i,k) */
int lim(i,j,k) int i, j, k;
    {if (i>=j) return i; if (k<=j) return k; return j;}

/* report Wolfram rule number or variant */
unsigned long int wolnum(c) char c; {
unsigned long int  k;
int       i0, i1, i2, i3;
  k=0;
  switch (c) {
    case 'w':
      for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=KK*k+(ascrule[i0][i1][i2][i3]-'0');
      break;
    case 'c':
      for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=KK*k+((ascrule[(i0+1)%KK][(i1+1)%KK][(i2+1)%KK][(i3+1)%KK]-'0')+1)%KK;
      break;
    case 'r':
      for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=KK*k+(ascrule[i3][i2][i1][i0]-'0');
      break;
    case 'b':
      for (i3=KK-1; i3>=0; i3--)
      for (i2=KK-1; i2>=0; i2--)
      for (i1=KK-1; i1>=0; i1--)
      for (i0=KK-1; i0>=0; i0--)
        k=KK*k+((ascrule[(i3+1)%KK][(i2+1)%KK][(i1+1)%KK][(i0+1)%KK]-'0')+1)%KK;
      break;
    default: k=0; break;}
        return k;
}

/* display the rule number on the screen */
scrrul() {int i0, i1, i2, i3;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    videoputc(ascrule[i0][i1][i2][i3],1);
  }

/* display the rule number on the screen in glorious technicolor */
cscrrul() {int i, i0, i1, i2, i3;
  i=6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++) {
  videocursor(0,3,i++);
  videocattr(0,ascrule[i0][i1][i2][i3],auxrule[i0][i1][i2][i3]==' '?3:6,1);
  } }

/* change decimal totalistic rule to sum values */
nutowo1(x) int x; {int i;
for (i=0; i<TS; i++) {prule1[i]=x%2+'0'; x/=2;}}

/* change Wolfram rule number to decimal rule number */
int wotonu1(i) int i; {int r;
  if (i<TS) r=(prule1[i]-'0')+2*totonu(i+1); else r=0;
  return r; }

/* change decimal number to Wolfram rule number */
nutowo2(x) int x; {int i;
for (i=0; i<TS; i++) {prule2[i]=x%2+'0'; x/=2;}}

/* change sum values to decimal totalistic rule */
int wotonu2(i) int i; {int r;
  if (i<TS) r=(prule2[i]-'0')+2*totonu(i+1); else r=0;
  return r; }

/* turn ascrule into its Wolfram number and place on screen at (i,j) */
woruno(i,j) int i, j; {
char xx[10];
int  k, l, i0, i1;
  k=0;
  for (i0=KK-1; i0>=0; i0--)
  for (i1=KK-1; i1>=0; i1--)
    k=3*k+(ascrule[i0][i1]-'0');
  k=sprintf(xx,"%3d",k);
  videocursor(0,i,j);
  for (l=0; l<k; l++) videoputc(xx[l],1);
}

void rule5t(){
    ranrul();
//    xtoasc(((int)random()%NX));
    ranlin();					/* random initial array */
}

void savearry() {int i; for (i=0; i<AL; i++) arr3[i]=arr1[i];}
void renewarry() {int i; for (i=0; i<AL; i++) arr1[i]=arr3[i];}

int  bdos(){}

/* end nxlcau5t.c */
