
/* nxprob5t.c						*/
/* program for lcau5t.c option 't'			*/
/* calculate probabilities related to evolution		*/
/* Harold V. McIntosh, 10 August 1987			*/
/* 29 February 1988 - adapted from LCA21 to LCA22	*/
/* 30 March 1989 - n, N for totalistic statistics	*/
/* 15 September 1989 - entropy calculations		*/
/* 14 October 1989 - adapted to radius 3/2		*/

/* references:							*/
/*								*/
/* W. John Wilbur, David J. Lipman and Shihab A. Shamma		*/
/* On the prediction of local patterns in cellular automata	*/
/* Physica 19D 397-410 (1986)					*/ 
/*								*/
/* Howard A. Gutowitz, Jonathan D. Victor and Bruce W. Knight	*/
/* Local structure theory for cellular automata			*/
/* Physica 28D 18-48 (1987)					*/

/*  Copyright (C) 1987  */
/*  Copyright (C) 1988  */
/*  Copyright (C) 1989  */
/*  Copyright (C) 1994  */
/*  Harold V. McIntosh  */

# include <math.h>
# include "nxlcau5t.h"
#import <dpsclient/psops.h>
#import <dpsclient/wraps.h>

extern char   arr1[AL];
extern char   ascrule[KK][KK][KK][KK];
extern int    binrule[KK][KK][KK][KK];

# define BER	    0	/* row (from bottom) for Bernstein p.	*/
# define BEC	  205	/* column for Bernstein Polynomial	*/
# define BROW	   13   /* row for bar charts			*/
# define EROW	   22   /* row for evolution synopsis		*/
# define TROW      89   /* row for t-triangle			*/
# define TCOL     205   /* column for t-triangle		*/

/* edit the probability screen	*/
edtri() {char c;
//  videomode(COLGRAF);
//  videopalette(YELREGR);
//  videotrgrid(TROW,TCOL,10,3);

  while (0<1) {}   /* end while  */
}      /* end edtri  */

/* show n-block probabilities */
void nblcur(n) int n; {switch (n) {
//    case '1': oneblfreq(0,300,48); break;
//    case '2': twoblfreq(0,300,48); break;
//    case '3': thrblfreq(0,300,48); break;
//    case '4': foublfreq(0,300,48); break;
//    case '5': fivblfreq(0,300,48); break;
//    case '6': sixblfreq(0,300,48); break;
//    case '7': svnblfreq(0,300,48); break;
    default: break;}
}

/* show menu */
trmenu() {
  videocursor(0,BROW,0);
  printf("a,h,H  - a priori estimates, entropy\n");
  printf("m,M,g,G  sample evolution\n");
  printf("s      - graph probabilities in stereo\n");
  printf("t,T    - graph mean field probabilities\n");
  printf("12345  - n-block bar charts\n");
  printf("+-     - change color pallette\n");
  printf("e      - 16 lines evolution\n");
  printf("z/?    - clear panel, screen; show menu\n");
  printf("<cr>   - exit\n");
}

/* show twelve lines of evolution at top of screen */
pevolve() {int i, j;
  videoscroll(EROW,0,EROW+1,40,0,0);
  asctobin();
  for (j=8*EROW; j<8*(EROW+2)-3; j++) {
    for (i=0; i<AL; i++) videodot(j,i,arr1[i]);
    onegen(AL);
    }
}

/* Clear a space for the n-block frequencies */
nblclr() {videoscroll(BROW,0,BROW+8,40,0,0);}

/* display state frequencies in arr1 as a dot in triangle at (u,v) */
/* n = generations of evolution					   */
/* (u,v) = corner of triangle					   */
/* l = color of dot						   */

lifreq(n,u,v,l) int n, u, v, l; {
int i, ii;
int stat[KK];
double staf[KK], s;

s=1.0/((double)(AL));
asctobin();
for (ii=0; ii<n; ii++) {
  onegen(AL);
  for (i=0; i<KK; i++) stat[i]=0;
  for (i=0; i<AL; i++) (stat[arr1[i]])++;
  for (i=0; i<KK; i++) staf[i]=s*((double)stat[i]);
  videotrdot(staf,l);
  }
}


/* bdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
bdot(i,j,l) int i, j, l; {videodot(199-BER-j,BEC+i,l);}

/* pdot(i,j,l) - place polkadot on Bernstein screen at (i,j) in color l */
pdot(i,j,l) int i, j, l; {videodot(199-BER-j,BEC+i,l+i%2);}

/* plot a single point on a triangular grid */

videotrdot(x,l) double x[KK]; int l; {double s, y, z;
s=199.0/(x[0]+x[1]+x[2]); y=x[1]*s; z=x[2]*s;
videodot((int)(0.433*z),(int)(0.250*(y+y+z)),l);
}

/* plot a stereo point on a triangular grid */
/* (u,v) screen location lower left corner  */
/* (x,y,z) triangular coordinates	    */
/* t       height			    */
/* l       color			    */

videostdot(u,v,x,t,l) double x[KK], t; int u, v, l; {double s, y, z;
s=199.0/(x[0]+x[1]+x[2]); y=x[1]*s; z=x[2]*s;
videodot(u-(int)(0.433*z),v+(int)(0.250*(y+y+z+t)),l);
}

/* set up a triangular gridwork */
/* (u,v) = corner of triangle	*/
/* n = number of rulings	*/
/* l = color of lines		*/

videotrgrid(u,v,n,l) int u, v, n, l; {
int i, j;
double a[KK], en;
if(n==0) return;
en=(double)n;
for (i=0; i<=n;   i++) {
for (j=0; i+j<=n; j++) {
  a[0]=((double)(i)/en);
  a[1]=((double)(j)/en);
  a[2]=1.0-a[0]-a[1];
  videotrdot(a,l);
  }}
}

/* plot a single line on a triangular grid */

videotrline(t,l) double t[KK]; int l; {double s, y, z;
s=400.0/(t[0]+t[1]+t[2]); y=t[1]*s; z=t[2]*s;
// videodot((int)(0.433*z),(int)(0.250*(y+y+z)),l);
if (l==0) PSmoveto(0.250*(y+y+z),400.0-0.433*z);
if (l==1) PSlineto(0.250*(y+y+z),400.0-0.433*z);
}

/* plot a single line on a triangular grid */

videotrli(a,b,c,l) double a, b, c; int l; {double s, y, z;
s=400.0/(a+b+c); y=b*s; z=c*s;
if (l==0) PSmoveto(0.250*(y+y+z),173.0-0.433*z);
if (l==1) PSlineto(0.250*(y+y+z),173.0-0.433*z);
}

/* plot a single line on a triangular grid */

videotrxli(a,b,c,l) double a, b, c; int l; {double s, y, z;
s=200.0/(a+b+c); y=b*s; z=c*s;
if (l==0) PSmoveto(150.0+0.250*(y+y+z),260.0-0.433*z);
if (l==1) PSlineto(150.0+0.250*(y+y+z),260.0-0.433*z);
}

/* set up a line grid */

linegrid(n) int n; {int i; double p, dp;
dp=1.0/((double) n);
  for (i=0, p=0.0; i<n; i++, p+=dp) {
    videotrli(0.0, p, 1.0-p, 0);
    videotrli(1.0-p, p, 0.0, 1);
    }
  for (i=0, p=0.0; i<n; i++, p+=dp) {
    videotrli(p, 1.0-p, 0.0, 0);
    videotrli(p, 0.0, 1.0-p, 1);
    }
  for (i=0, p=0.0; i<n; i++, p+=dp) {
    videotrli(1.0-p, 0.0, p, 0);
    videotrli(0.0, 1.0-p, p, 1);
    }
  }

/* set up an extended line grid */

xlinegrid(n) int n; {int i; double p, dp;
dp=1.0/((double) n);
  for (i=0, p=0.0; i<n; i++, p+=dp) {
    videotrxli(0.0, p, 1.0-p, 0);
    videotrxli(1.0-p, p, 0.0, 1);
    }
  for (i=0, p=0.0; i<n; i++, p+=dp) {
    videotrxli(p, 1.0-p, 0.0, 0);
    videotrxli(p, 0.0, 1.0-p, 1);
    }
  for (i=0, p=0.0; i<n; i++, p+=dp) {
    videotrxli(1.0-p, 0.0, p, 0);
    videotrxli(0.0, 1.0-p, p, 1);
    }
  }

/* option 'n' (general neighborhood) */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

void gnlsum(ll,mm,nn) int ll, mm, nn; {
int    i0, i1, i2;
int    ii, k, m, n, l, b[KK][KK][KK];
double em, en, f, bf[KK][KK][KK], bft[KK][KK][KK];

asctobin();
em=((double)(mm));
// f=1.0;
f=115.0/((double)(AL));
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	bft[i0][i1][i2]=0.0;
k=0;
for (ii=0; ii<nn; ii++) {
  onegen(mm);
  gnrsum(mm,b);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	bf[i0][i1][i2]=((double)(b[i0][i1][i2]));
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	bft[i0][i1][i2]+=bf[i0][i1][i2];
  m=0; videodot(ll+ii,m++,3);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) {
    n=(int)(f*bf[i0][i1][i2]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i2%KK);
    }
  videodot(ll+ii,m++,3);
  k++;
  }  /* end for-ii */
  en=((double)(k));
//  m=0; videodot(ll+nn,m++,3);
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) {
    n=(int)(0.1*(bft[i0][i1][i2]/en));
//    if (n>0) for (l=0; l<n; l++) videodot(ll+nn,m++,i0%KK);
    }
//  videodot(ll+nn,m++,3);
// for (i=0; i<TS; i++) bft[i]/=en*em;
// videocursor(0,BROW+7,0);
// for (i=0; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
}

/* option 'N' */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/

nbrsum(ll,mm,nn) int ll, mm, nn; {
int ii, jj, i, k, m, n, l, b[TS];
double em, en, kk, bf[TS], bft[TS];

asctobin();
em=((double)(mm));
en=((double)(nn));
for (i=0; i<TS; i++) bft[i]=0.0;
k=0;
for (jj=0; jj<nn; jj++) {
  for (i=0; i<TS; i++) bf[i]=0.0;
  for (ii=0; ii<nn; ii++) {
    onegen(mm);
    onesum(mm,TS,b);
    for (i=0; i<TS; i++) bf[i]+=((double)(b[i]));
    }  /* end for-ii */
  m=0; videodot(ll+jj,m++,3);
  for (i=0; i<TS; i++) {
    n=(int)(bf[i]/en);
    if (n>0) for (l=0; l<n; l++) videodot(ll+jj,m++,i%2+1);
    }
  videodot(ll+jj,m++,3);
  for (i=0; i<TS; i++) bft[i]+=bf[i];
  k++;
  } /* end for-jj */
kk=((double)(k));
videodot(ll+jj,m++,3);
for (i=0; i<TS; i++) bft[i]/=kk*en*em;
videocursor(0,BROW+7,0);
for (i=0; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
/* videocursor(0,BROW+8,0);				*/
/* for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);	*/
}

/* option 'm' */
/* "Monte Carlo" determination of probabilities */
/* n = mumber of generations			*/
/* l = color of dot				*/

moncar(n,l) int n, l; {
int i, j, k, b[KK], bb[KK][KK];
double bf[KK], bbf[KK][KK];
double a[KK], aa[KK], d[KK], el, en, t, tt, s;

nblclr();
/* gfram(BROW,0,100); */
asctobin();
el=1.0/((double)AL);
en=1.0/((double)n);
for (i=0; i<KK; i++) {a[i]=0.0; aa[i]=0.0;}
for (k=0; k<n; k++) {
  onegen(AL);
  for (i=0; i<KK; i++) b[i]=0;
  for (i=0; i<AL; i++) b[arr1[i]]++;
  for (i=0; i<KK; i++) bf[i]=((double)(b[i]))*el;
  for (i=0; i<KK; i++) {a[i]+=bf[i]; aa[i]+=bf[i]*bf[i];}
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) bb[i][j]=0;
  for (i=1; i<AL; i++) bb[arr1[i-1]][arr1[i]]++;
  bb[arr1[AL-1]][arr1[0]]++; 
  for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
    bbf[i][j]=((double)(bb[i][j]))*el;
  videodot(199-(int)(100.0*bbf[1][1]),BROW+(int)(100.0*bf[1]),l);
  };
for (i=0; i<KK; i++) {a[i]*=en; aa[i]*=en;}
videocursor(0,BROW+1,0);
s=1.0/log((double)KK);
t=0.0; tt=0.0;
for (i=0; i<KK; i++) t+=bf[i]*log(bf[i]);
for (i=0; i<KK; i++) d[i]=sqrt(aa[i]-a[i]*a[i]);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) tt+=bbf[i][j]*log(bbf[i][j]);
printf("(Monte Carlo) \n"); 
printf("lambda       = %5.3f\n",1.0-bf[0]);
printf("entropy      = %5.3f\n",-t*s);
printf("pair entropy = %5.3f\n",-0.5*tt*s);
if (n>1) printf("average      = %5.3f, %5.3f\n",a[0],a[1]);
if (n>1) printf("deviation    = %5.3f, %5.3f\n",d[0],d[1]);
printf("frequencies ");
for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,bf[i]);
printf("\n");
for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
  printf("%1d%1d:%5.3f ",i,j,bbf[i][j]);
printf("\n");
}

/* option 'n' */
/* empirical distribution of sums within a neighborhood */
/* ll - initial line	*/
/* mm - number of cells	*/
/* nn - number of lines	*/
neisum(ll,mm,nn) int ll, mm, nn; {
int ii, i, k, m, n, l, b[TS];
double em, en, bf[TS], bft[TS];

asctobin();
em=((double)(mm));
for (i=0; i<TS; i++) bft[i]=0.0;
k=0;
for (ii=0; ii<nn; ii++) {
  onegen(mm);
  onesum(mm,TS,b);
  for (i=0; i<TS; i++) bf[i]=((double)(b[i]));
  for (i=0; i<TS; i++) bft[i]+=bf[i];
  m=0; videodot(ll+ii,m++,3);
  for (i=0; i<TS; i++) {
    n=(int)(bf[i]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i%2);
    }
  videodot(ll+ii,m++,3);
  k++;
  }  /* end for-ii */
  en=((double)(k));
  m=0; videodot(ll+nn,m++,3);
  for (i=0; i<TS; i++) {
    n=(int)(bft[i]/en);
    if (n>0) for (l=0; l<n; l++) videodot(ll+nn,m++,i%2+1);
    }
  videodot(ll+nn,m++,3);
for (i=0; i<TS; i++) bft[i]/=en*em;
videocursor(0,BROW+7,0);
for (i=0; i<5; i++) printf("%1d:%5.3f ",i,bft[i]);
videocursor(0,BROW+8,0);
for (i=5; i<TS; i++) printf("%1d:%5.3f ",i,bft[i]);
}

/* gather general neighborhood sums for one generation */
/* mm = number of cells         */
/* b[KK][KK][KK]  = sum counts  */
void gnrsum(mm,b) int mm, b[KK][KK][KK]; {int i0, i1, i2, i;
  for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) for (i2=0; i2<KK; i2++) 
	b[i0][i1][i2]=0;
  b[arr1[mm-1]][arr1[0]][arr1[1]]++;
  for (i=1; i<mm-2; i++) b[arr1[i-1]][arr1[i]][arr1[i+1]]++;
  b[arr1[mm-2]][arr1[mm-1]][arr1[0]]++;
}

/* gather totalistic neighborhood sums for one generation */
/* mm = number of cells */
/* b[cc]  = sum counts  */
onesum(mm,cc,b) int mm, cc, *b; {int i, j, k;
  for (i=0; i<cc; i++) b[i]=0;
  k=arr1[mm-2]+arr1[mm-1]+arr1[0]+arr1[1]+arr1[2]; b[k]++;
  k=arr1[mm-1]+arr1[0]+arr1[1]+arr1[2]+arr1[3]; b[k]++;
  for (i=2; i<mm-3; i++) {k=0; for (j=i-2; j<i+3; j++) k+=arr1[j]; b[k]++;}
  k=arr1[mm-4]+arr1[mm-3]+arr1[mm-2]+arr1[mm-1]+arr1[0]; b[k]++;
  k=arr1[mm-3]+arr1[mm-2]+arr1[mm-1]+arr1[0]+arr1[1]; b[k]++;
}

/* make a grid for the fixpoint graph  */
/* n     = dimension of grid           */
ggrid(n) int n; {int i, j;
for (i=0; i<=n; i+=10) for (j=0; j<=n; j+=10) bdot(i,j,3);}

/* put a diagonal in the fixpoint graph */
gdiag(n) int n; {int i; for (i=0; 2*i<n; i++) bdot(2*i,2*i,3);}

/* plot self-consistent probability graph */
bpanel()  {ggrid(100); gdiag(100); bgraf(100);}

/* plot self-consistent graph in Langton's parameter */
lpanel()  {ggrid(100); gdiag(100); lgraf(100);}

/* graph Bernstein polynomial */
bgraf(n) int n; {
int    i, j;
double bern(), el, em, en, p, q, t[KK];
en=(double)(n); em=1.0/en; el=1.0/((double)(KK-1));
for (p=0.0, i=0; i<n; p+=em, i++) {
  q=el*(1.0-p); for (j=1; j<KK; j++) t[j]=q;
  t[0]=p; pdot(i,(int)(en*bern(0,t)),1);
  for (j=1; j<KK; j++) {t[j-1]=q; t[j]=p; bdot(i,(int)(en*bern(j,t)),1+j%3);}
  }
}

/* graph composite Bernstein multinomial */
dbgraf(n) int n; {
int    i;
double dbern(), el, em, en, p, q, t[KK];
en=(double)(n); em=1.0/en; el=1.0/((double)(KK-1));
for (p=0.0, i=0; i<n; p+=em, i++) {
  q=el*(1.0-p); t[1]=t[2]=q;
  t[0]=p; bdot(i,(int)(en*dbern(0,t)),3);
  t[0]=q; t[1]=p; bdot(i,(int)(en*dbern(1,t)),1);
  t[1]=q; t[2]=p; bdot(i,(int)(en*dbern(2,t)),2);
  }
}

/* graph Langton's parameter */
lgraf(n) int n; {int i, j;
double bern(), el, em, en, p, q, t[KK];
en=(double)(n); em=1.0/en; el=1.0/((double)(KK-1));
for (p=0.0, i=0; i<n; p+=em, i++) {
  t[0]=p; q=el*(1.0-p); for (j=1; j<KK; j++) t[j]=q;
  pdot(n-i,n-(int)(en*bern(0,t)),1);
  }
}

/* evaluate the Bernstein polynomial for component i at point t[KK] */
double bern(i,t) int i; double t[KK]; {
int i0, i1, i2, i3; double a, b, c, d;
asctobin();
for (i0=0, a=0.0; i0<KK; i0++) {
  for (i1=0, b=0.0; i1<KK; i1++) {
    for (i2=0, c=0.0; i2<KK; i2++) {
      for (i3=0, d=0.0; i3<KK; i3++)
	  if (binrule[i0][i1][i2][i3]==i) d+=t[i3];
      c+=d*t[i2];}
    b+=c*t[i1];}
  a+=b*t[i0];}
return a;
}

/* evaluate composite Bernstein polynomial for component i at point t[KK] */
double dbern(i,t) int i; double t[KK]; {
int    i0, i1, i2, i3, i4, i5, i6;
int    j0, j1, j2, j3;
double a, b, c, d, e, f, g;
asctobin();
for (i0=0, a=0.0; i0<KK; i0++) {
  for (i1=0, b=0.0; i1<KK; i1++) {
    for (i2=0, c=0.0; i2<KK; i2++) {
      for (i3=0, d=0.0; i3<KK; i3++) {
        for (i4=0, e=0.0; i4<KK; i4++) {
          for (i5=0, f=0.0; i5<KK; i5++) {
            for (i6=0, g=0.0; i6<KK; i6++) {
	      j0=binrule[i0][i1][i2][i3];
	      j1=binrule[i1][i2][i3][i4];
	      j2=binrule[i2][i3][i4][i5];
	      j3=binrule[i3][i4][i5][i6];
              if (binrule[j0][j1][j2][j3]==i) g+=t[i6];
	      }
            f+=g*t[i5];}
          e+=f*t[i4];}
        d+=e*t[i3];}
      c+=d*t[i2];}
    b+=c*t[i1];}
  a+=b*t[i0];}
return a;
}

/* graph the entropy over a triangle */
penry(n) int n; {
int    i, j, lnco();
double a[KK], t, em, en;
if (n==0) return;
em=1.0/log(3.0);
en=1.0/((double)(n));
for (i=0; i<=n; i++) {
for (j=0; i+j<=n; j++) {
  t=0.0;
  a[0]=((double)(i))*en; if (a[0]>0.005) t+=a[0]*log(a[0]);
  a[1]=((double)(j))*en; if (a[1]>0.005) t+=a[1]*log(a[1]);
  a[2]=1.0-a[0]-a[1]; if (a[2]>0.005) t+=a[2]*log(a[2]);
  videotrdot(a,lnco(-t*em,i));
  }}
}

/* graph the probability differential for state l over 	*/
/* an extended triangle 	 			*/

xpdiff(n,l) int n, l; {
int    i, j, sqco();
double a[KK], en, t, sqsu();
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=-n; i<=3*n; i++) {
for (j=-n; i+j<=2*n; j++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=1.0-a[0]-a[1];
  t=sqsu(a);
// printf("%6.3f, %6.3f, %6.3f, - %6.3f\n",a[0],a[1],a[2],t); 
  videotrdot(a,sqco(t,i));
  }}
}

/* graph the probability differential for state l over a triangle */

pdiff(n,l) int n, l; {
int    i, j, sqco();
double a[KK], en, t, sqsu();
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=0; i<=n; i++) {
for (j=0; i+j<=n; j++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=1.0-a[0]-a[1];
  t=sqsu(a);
  videotrdot(a,sqco(t,i));
  }}
}

/* graph the probability differential for state l over a triangle */

pddiff(n,l) int n, l; {
int    i, j, sqco();
double a[KK], en, t, dsqsu();
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=0; i<=n; i++) {
for (j=0; i+j<=n; j++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=1.0-a[0]-a[1];
  t=dsqsu(a);
// printf("%6.3f, %6.3f, %6.3f, - %6.3f\n",a[0],a[1],a[2],t); 
  videotrdot(a,sqco(t,i));
  }}
}

/* stereo graph of probability differential for state l over a triangle */

spdiff(n,l) int n, l; {int i, j; double a[KK], en, t, sqsu();
if (n==0) return;
if (l!=6) return;
en=1.0/((double)(n));
for (i=0; i<=n; i++) {
for (j=0; i+j<=n; j++) {
  a[0]=((double)(i))*en;
  a[1]=((double)(j))*en;
  a[2]=1.0-a[0]-a[1];
  t=sqsu(a);
  videostdot(99,5,a,-12.0*t,sqco(t,i));
  videostdot(99,105,a,12.0*t,sqco(t,i));
  }}
}

/* calculate the squaresum of the three polynomials at a point */

double sqsu(x) double x[KK]; {double bern(), s, t; int i;
for (i=0, s=0.0; i<KK; i++) {t=bern(i,x)-x[i]; s+=t*t;}
return s;
}

/* calculate the squaresum of the three polynomials at a point  */
/* for composite evolution					*/

double dsqsu(t) double t[KK]; {double u, v, w, dbern();
u=dbern(0,t)-t[0];
v=dbern(1,t)-t[1];
w=dbern(2,t)-t[2];
return (u*u+v*v+w*w);
}

/* generate linearly compatible contour values */

int lnco(t,i) int i; double t; {int l;
if (t<0.125) l=0; else {
if (t<0.250) {if (i%2) l=0; else l=3;} else {
if (t<0.375) l=3; else {
if (t<0.500) {if (i%2) l=3; else l=2;} else {
if (t<0.625) l=2; else {
if (t<0.750) {if (i%2) l=2; else l=1;} else {
if (t<0.875) l=1; else {
	      if (i%2) l=1; else l=0;
   }}}}}}}
return l;
}

/* generate squaresum compatible contour values */

int sqco(t,i) int i; double t; {int l;
if (t<0.0075) l=0; else {
if (t<0.0750) {if (i%2) l=0; else l=3;} else {
if (t<0.1250) l=3; else {
if (t<0.2500) {if (i%2) l=3; else l=2;} else {
if (t<0.5000) l=2; else {
if (t<0.7500) {if (i%2) l=2; else l=1;} else {
if (t<1.0000) l=1; else {
	      if (i%2) l=1; else l=0;
   }}}}}}}
return l;
}

/* graph the frequencies of ascrule in color l	 */
/* put dot at coordinates (u,v)			 */
/* percentages evolving into each value		 */
/* percentages shifting - adds to more than 100% */
/* entropy and Langton's lambda = % nonquiescent */
/* display frequencies in ascrule in color l */

asfreq(l) int l; {
int    i, j, i0, i1, i2, i3;
int    stat[KK], stall, stal, star, starr;	/* statistic counts */
double staf[KK], t, pp;

videoscroll(BROW,0,BROW+8,40,0,0);
videocursor(0,BROW,0);
pp=1.0/((double)(KK*KK*KK*KK));
stall=0; stal=0; star=0; starr=0;
for (i=0; i<KK; i++) stat[i]=0;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
  j=ascrule[i0][i1][i2][i3]-'0';
  stat[j]++;
  if(j==i0) starr++;
  if(j==i1) star++;
  if(j==i2) stal++;
  if(j==i3) stall++;
  }}}}
videocursor(0,BROW,0);
for (i=0; i<KK; i++) staf[i]=((double)stat[i])*pp;
for (i=0; i<KK; i++) printf("%d freq - %5.2f\n",i,((double)stat[i])*pp);
printf("\n");
printf("lleft  - %5.2f\n",((double)stall)*pp);
printf("left   - %5.2f\n",((double)stal)*pp);
printf("right  - %5.2f\n",((double)star)*pp);
printf("rright - %5.2f\n",((double)starr)*pp);
j=199-(int)(100.0*staf[1]);
i=BROW+(int)(100.0*staf[1]);
if (j>=2) videodot(j-2,i,l);
if (i>=2) videodot(j,i-2,l);
videodot(j,i,1);
if (i<=318) videodot(j,i+2,l);
if (j<=198) videodot(j+2,i,l);
for (i=0, t=0.0; i<KK; i++) t+=staf[i]*log(staf[i]);
videocursor(0,BROW,20);
printf("lambda:   %5.3f",1.0-staf[0]);
videocursor(0,BROW+1,20);
printf("entropy:  %5.3f",-t/log((double)KK));
videocursor(0,BROW,0);
}

/* evaluate the parameters for two-block probabilities after one generation */

twoblock(z) double *z; {
int    i, i0, i1, i2, i3, i4;
int    j, j0, j1;
double w, x, y, p, q, pp;
double a[KK][KK], b[KK], c[KK][KK];

p=z[0]; q=1.0-p; pp=z[1];
a[1][1]=pp; a[1][0]=a[0][1]=p-pp; a[0][0]=1.0-2*p+pp;
b[1]=p; b[0]=q;
for (i=0; i<KK; i++) {for (j=0; j<KK; j++) c[i][j]=0.0;}
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
  j0=ascrule[i0][i1][i2][i3]-'0';
  j1=ascrule[i1][i2][i3][i4]-'0';
  x=a[i0][i1]*a[i1][i2]*a[i2][i3]*a[i3][i4];
  y=b[i1]*b[i2]*b[i3];
  w=0.0; if (y>0.00001) w=x/y;
  c[j0][j1]+=w;
  }}}}}
z[0]=c[1][0]+c[1][1]; z[1]=c[1][1];
}

/* graph the 2-block parameter differences to find self-consistent values */
/* nxn points 								  */

twoblockfreq(n) int n; {
int i, j, l;
double op[2], np[2], s, t, u, v;

/* gfram(0,0,100); */

s=1.0/((double)(n));
op[1]=s;
for (i=1; i<n; i++) {
  op[0]=s;
  for (j=1; j<n; j++) {
    np[0]=op[0]; np[1]=op[1]; twoblock(np);
    u=np[0]-op[0]; v=np[1]-op[1];
    t=u*u+v*v;
    if (t<0.0025) l=0; else {
    if (t<0.0675) {if((i+j)%2) l=0; else l=3;} else {
    if (t<0.3906) l=3; else {
    if (t<0.7500) {if((i+j)%2) l=3; else l=2;} else {
    if (t<1.5000) l=2; else {
    if (t<3.0000) {if((i+j)%2) l=2; else l=1;} else {
    if (t<6.0000) l=1; else {
                  if((i+j)%2) l=1; else l=0;
       }}}}}}}
    videodot(199-i,j,l);
    op[0]+=s;
    }  /* end for-j */
  op[1]+=s;
  }    /* end for-i */
}

/* evaluate the one-block probabilities after one generation */

onebl(x,a) double x[KK], a[KK]; {int i0, i1, i2, i3, j0;

for (j0=0; j0<KK; j0++) x[j0]=0.0;

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
  j0=ascrule[i0][i1][i2][i3]-'0';
  x[j0]+=a[i0]*a[i1]*a[i2]*a[i3];
  }}}}
}

/* iterate the 1-block parameters to find self-consistent values */
/* graph the iterative steps in bar-chart form			 */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

oneblfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i, l, m, n;
double op[KK], np[KK];
double d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i=0; i<KK; i++) {op[i]=s; for (l=0; l<n; l++) videodot(ll,m++,i);}

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  onebl(np,op);
  videodot(ll+ii,m++,3);
  for (i=0; i<KK; i++) {
    n=(int)(f*np[i]); if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i);
    d=op[i]-np[i];
    e+=d*d;
    op[i]=np[i];
    }
  videodot(ll+ii,m++,3);
  if (op[0]<=0.001) break;
  if (op[1]<=0.001) break;
  if (e<=0.0000001) break;
  }
videocursor(0,BROW+7,0);
printf("(1-block) "); 
for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,op[i]);
}

/* evaluate the two-block probabilities after one generation */

twobl(x,a) double x[KK][KK], a[KK][KK]; {
int    i0, i1, i2, i3, i4;
int    j0, j1;
double w, b[KK];

for (j0=0; j0<KK; j0++) {for (j1=0; j1<KK; j1++) x[j0][j1]=0.0;}

for (j0=0; j0<KK; j0++) {b[j0]=0.0; for (j1=0; j1<KK; j1++) b[j0]+=a[j0][j1];}

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
  j0=ascrule[i0][i1][i2][i3]-'0';
  j1=ascrule[i1][i2][i3][i4]-'0';
  w=a[i0][i1]*a[i1][i2]*a[i2][i3]*a[i3][i4];
  if (w!=0.0) w/=b[i1]*b[i2]*b[i3];
  x[j0][j1]+=w;
  }}}}}
}

/* iterate the 2-block parameters to find self-consistent values */
/* graph the iterative steps in bar-chart form			 */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

twoblfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i, j, l, m, n;
double op[KK][KK], np[KK][KK];
double b[KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i=0; i<KK; i++) {
for (j=0; j<KK; j++) {
  op[i][j]=s;
  for (l=0; l<n; l++) videodot(ll,m++,j);
  }}
videodot(ll,m++,3);

for (i=0; i<KK; i++) {b[i]=0.0; for (j=0; j<KK; j++) b[i]+=op[i][j];}
videodot(199-(int)(100.0*op[1][1]),(int)(100.0*b[1]),1);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  twobl(np,op);
  videodot(ll+ii,m++,3);
  for (i=0; i<KK; i++) {
  for (j=0; j<KK; j++) {
    n=(int)(f*np[i][j]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,j);
    d=op[i][j]-np[i][j];
    e+=d<0.0?-d:d;
    op[i][j]=np[i][j];
    }}
  videodot(ll+ii,m++,3);
  for (i=0; i<KK; i++) {b[i]=0.0; for (j=0; j<KK; j++) b[i]+=op[i][j];}
  videodot(199-(int)(100.0*op[1][1]),(int)(100.0*b[1]),2);
  if (e<=0.0001) break;
  }
videocursor(0,BROW+7,0);
printf("(2-block) "); 
for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,b[i]);
videocursor(0,BROW+8,0);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
		printf("%1d%1d:%5.3f ",i,j,op[i][j]);
videodot(199-(int)(100.0*op[1][1]),(int)(100.0*b[1]),3);
videodot(199-(int)(100.0*op[1][1]),BROW+(int)(100.0*b[1]),1);
}

/* evaluate the three-block probabilities after one generation */

thrbl(x,a) double x[KK][KK][KK], a[KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5;
int    j0, j1, j2;
double w, b[KK][KK];

for (j0=0; j0<KK; j0++) {
for (j1=0; j1<KK; j1++) {
  for (j2=0; j2<KK; j2++) x[j0][j1][j2]=0.0;
  b[j0][j1]=0.0;
  for (j2=0; j2<KK; j2++) b[j0][j1]+=a[j0][j1][j2];  
  }}

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
for (i5=0; i5<KK; i5++) {
  j0=ascrule[i0][i1][i2][i3]-'0';
  j1=ascrule[i1][i2][i3][i4]-'0';
  j2=ascrule[i2][i3][i4][i5]-'0';
  w=a[i0][i1][i2]*a[i1][i2][i3]*a[i2][i3][i4]*a[i3][i4][i5];
  if (w!=0.0) w/=b[i1][i2]*b[i2][i3]*b[i3][i4];
  x[j0][j1][j2]+=w;
  }}}}}}
}

/* iterate the 3-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

thrblfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i, j, k, l, m, n;
double op[KK][KK][KK], np[KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i=0; i<KK; i++) {
for (j=0; j<KK; j++) {
for (k=0; k<KK; k++) {
  op[i][j][k]=s;
  for (l=0; l<n; l++) videodot(ll,m++,k);
  }}}
videodot(ll,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  thrbl(np,op);
  videodot(ll+ii,m++,3);
  for (i=0; i<KK; i++) {
  for (j=0; j<KK; j++) {
  for (k=0; k<KK; k++) {
    n=(int)(f*np[i][j][k]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,k);
    d=op[i][j][k]-np[i][j][k];
    e+=d<0.0?-d:d;
    op[i][j][k]=np[i][j][k];
    }}}
  videodot(ll+ii,m++,3);
  if (e<=0.0001) break;
  }

for (i=0; i<KK; i++) {
  b[i]=0.0;
  for (j=0; j<KK; j++) {
  for (k=0; k<KK; k++) {
    b[i]+=op[i][j][k];
    }} }
videocursor(0,BROW+7,0);
printf("(3-block) "); 
for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,b[i]);

for (i=0; i<KK; i++) {
for (j=0; j<KK; j++) {
  bb[i][j]=0.0;
  for (k=0; k<KK; k++) {
    bb[i][j]+=op[i][j][k];
    }} }

videocursor(0,BROW+8,0);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
  printf("%1d%1d:%5.3f ",i,j,bb[i][j]);
videodot(199-(int)(100.0*bb[1][1]),BROW+(int)(100.0*b[1]),2);
}

/* evaluate the four-block probabilities after one generation */

foubl(x,a) double x[KK][KK][KK][KK], a[KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6;
int    j0, j1, j2, j3;
double w, b[KK][KK][KK];

for (j0=0; j0<KK; j0++) {
for (j1=0; j1<KK; j1++) {
for (j2=0; j2<KK; j2++) {
  for (j3=0; j3<KK; j3++) x[j0][j1][j2][j3]=0.0;
  b[j0][j1][j2]=0.0;
  for (j3=0; j3<KK; j3++) b[j0][j1][j2]+=a[j0][j1][j2][j3];  
  }}}

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
for (i5=0; i5<KK; i5++) {
for (i6=0; i6<KK; i6++) {
  j0=ascrule[i0][i1][i2][i3]-'0';
  j1=ascrule[i1][i2][i3][i4]-'0';
  j2=ascrule[i2][i3][i4][i5]-'0';
  j3=ascrule[i3][i4][i5][i6]-'0';
  w=a[i0][i1][i2][i3];
  w*=a[i1][i2][i3][i4];
  w*=a[i2][i3][i4][i5];
  w*=a[i3][i4][i5][i6];
  if (w!=0.0) w/=b[i1][i2][i3]*b[i2][i3][i4]*b[i3][i4][i5];
  x[j0][j1][j2][j3]+=w;
  }}}}}}}
}

/* iterate the 4-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

foublfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i0, i1, i2, i3, l, m, n;
double op[KK][KK][KK][KK], np[KK][KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
  op[i0][i1][i2][i3]=s;
  for (l=0; l<n; l++) videodot(ll,m++,i3);
  }}}}
videodot(ll,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  foubl(np,op);
  videodot(ll+ii,m++,3);
  for (i0=0; i0<KK; i0++) {
  for (i1=0; i1<KK; i1++) {
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
    n=(int)(f*np[i0][i1][i2][i3]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i3);
    d=op[i0][i1][i2][i3]-np[i0][i1][i2][i3];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3]=np[i0][i1][i2][i3];
    }}}}
  videodot(ll+ii,m++,3);
  if (e<=0.0001) break;
  }

for (i0=0; i0<KK; i0++) {
  b[i0]=0.0;
  for (i1=0; i1<KK; i1++) {
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
    b[i0]+=op[i0][i1][i2][i3];
    }}} }

videocursor(0,BROW+7,0);
printf("(4-block) "); 
for (i0=0; i0<KK; i0++) printf("%2d:%5.3f ",i0,b[i0]);

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
    bb[i0][i1]+=op[i0][i1][i2][i3];
    }} }}

videocursor(0,BROW+8,0);
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
  printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);
videodot(199-(int)(100.0*bb[1][1]),BROW+(int)(100.0*b[1]),2);
}

/* evaluate the five-block probabilities after one generation */

fivbl(x,a) double x[KK][KK][KK][KK][KK], a[KK][KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6, i7;
int    j0, j1, j2, j3, j4;
double w, b[KK][KK][KK][KK];

for (j0=0; j0<KK; j0++) {
for (j1=0; j1<KK; j1++) {
for (j2=0; j2<KK; j2++) {
for (j3=0; j3<KK; j3++) {
  for (j4=0; j4<KK; j4++) x[j0][j1][j2][j3][j4]=0.0;
  b[j0][j1][j2][j3]=0.0;
  for (j4=0; j4<KK; j4++) b[j0][j1][j2][j3]+=a[j0][j1][j2][j3][j4];  
  }}}}

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
for (i5=0; i5<KK; i5++) {
for (i6=0; i6<KK; i6++) {
for (i7=0; i7<KK; i7++) {
  j0=ascrule[i0][i1][i2][i3]-'0';
  j1=ascrule[i1][i2][i3][i4]-'0';
  j2=ascrule[i2][i3][i4][i5]-'0';
  j3=ascrule[i3][i4][i5][i6]-'0';
  j4=ascrule[i4][i5][i6][i7]-'0';
  w=a[i0][i1][i2][i3][i4];
  w*=a[i1][i2][i3][i4][i5];
  w*=a[i2][i3][i4][i5][i6];
  w*=a[i3][i4][i5][i6][i7];
  if (w!=0.0) w/=b[i1][i2][i3][i4]*b[i2][i3][i4][i5]*b[i3][i4][i5][i6];
  x[j0][j1][j2][j3][j4]+=w;
  }}}}}}}}
}

/* iterate the 5-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

fivblfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i0, i1, i2, i3, i4, l, m, n;
double op[KK][KK][KK][KK][KK], np[KK][KK][KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK*KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
  op[i0][i1][i2][i3][i4]=s;
  for (l=0; l<n; l++) videodot(ll,m++,i4);
  }}}}}
videodot(ll,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  fivbl(np,op);
  videodot(ll+ii,m++,3);
  for (i0=0; i0<KK; i0++) {
  for (i1=0; i1<KK; i1++) {
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++) {
    n=(int)(f*np[i0][i1][i2][i3][i4]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i4);
    d=op[i0][i1][i2][i3][i4]-np[i0][i1][i2][i3][i4];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3][i4]=np[i0][i1][i2][i3][i4];
    }}}}}
  videodot(ll+ii,m++,3);
  if (e<=0.0001) break;
  }
for (i0=0; i0<KK; i0++) {
  b[i0]=0.0;
  for (i1=0; i1<KK; i1++) {
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++) {
    b[i0]+=op[i0][i1][i2][i3][i4];
    }}}} }
videocursor(0,BROW+7,0);
printf("(5-block) "); 
for (i0=0; i0<KK; i0++) printf("%2d:%5.3f ",i0,b[i0]);

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++) {
    bb[i0][i1]+=op[i0][i1][i2][i3][i4];
    }} }}}

videocursor(0,BROW+8,0);
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
  printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);
videodot(199-(int)(100.0*bb[1][1]),BROW+(int)(100.0*b[1]),2);
}

/* evaluate the six-block probabilities after one generation */

sixbl(x,a) double x[KK][KK][KK][KK][KK][KK], a[KK][KK][KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6, i7, i8;
int    j0, j1, j2, j3, j4, j5;
double w, b[KK][KK][KK][KK][KK];

for (j0=0; j0<KK; j0++) {
for (j1=0; j1<KK; j1++) {
for (j2=0; j2<KK; j2++) {
for (j3=0; j3<KK; j3++) {
for (j4=0; j4<KK; j4++) {
  for (j5=0; j5<KK; j5++) x[j0][j1][j2][j3][j4][j5]=0.0;
  b[j0][j1][j2][j3][j4]=0.0;
  for (j5=0; j5<KK; j5++) b[j0][j1][j2][j3][j4]+=a[j0][j1][j2][j3][j4][j5];  
  }}}}}

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
for (i5=0; i5<KK; i5++) {
for (i6=0; i6<KK; i6++) {
for (i7=0; i7<KK; i7++) {
for (i8=0; i8<KK; i8++) {
  j0=ascrule[i0][i1][i2][i3]-'0';
  j1=ascrule[i1][i2][i3][i4]-'0';
  j2=ascrule[i2][i3][i4][i5]-'0';
  j3=ascrule[i3][i4][i5][i6]-'0';
  j4=ascrule[i4][i5][i6][i7]-'0';
  j5=ascrule[i5][i6][i7][i8]-'0';
  w=a[i0][i1][i2][i3][i4][i5];
  w*=a[i1][i2][i3][i4][i5][i6];
  w*=a[i2][i3][i4][i5][i6][i7];
  w*=a[i3][i4][i5][i6][i7][i8];
  if (w!=0.0) w/=b[i1][i2][i3][i4][i5]*b[i2][i3][i4][i5][i6]
	*b[i3][i4][i5][i6][i7];
  x[j0][j1][j2][j3][j4][j5]+=w;
  }}}}}}}}}
}

/* iterate the 6-block parameters to find self-consistent values */
/* ll - initial line	*/
/* mm - length of line	*/
/* nn - number of lines	*/

sixblfreq(ll,mm,nn) int ll, mm, nn; {
int    ii, i0, i1, i2, i3, i4, i5, l, m, n;
double op[KK][KK][KK][KK][KK][KK];
double np[KK][KK][KK][KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK*KK*KK));
n=(int)(f*s);
videodot(ll,m++,3);
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
for (i5=0; i5<KK; i5++) {
  op[i0][i1][i2][i3][i4][i5]=s;
  for (l=0; l<n; l++) videodot(ll,m++,i5);
  }}}}}}
videodot(ll,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  sixbl(np,op);
  videodot(ll+ii,m++,3);
  for (i0=0; i0<KK; i0++) {
  for (i1=0; i1<KK; i1++) {
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++) {
  for (i5=0; i5<KK; i5++) {
    n=(int)(f*np[i0][i1][i2][i3][i4][i5]);
    if (n>0) for (l=0; l<n; l++) videodot(ll+ii,m++,i5);
    d=op[i0][i1][i2][i3][i4][i5]-np[i0][i1][i2][i3][i4][i5];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3][i4][i5]=np[i0][i1][i2][i3][i4][i5];
    }}}}}}
  videodot(ll+ii,m++,3);
  if (e<=0.0001) break;
  }
for (i0=0; i0<KK; i0++) {
  b[i0]=0.0;
  for (i1=0; i1<KK; i1++) {
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++) {
  for (i5=0; i5<KK; i5++) {
    b[i0]+=op[i0][i1][i2][i3][i4][i5];
    }}}}} }
videocursor(0,BROW+7,0);
printf("(6-block) "); 
for (i0=0; i0<KK; i0++) printf("%2d:%5.3f ",i0,b[i0]);

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++) {
  for (i5=0; i5<KK; i5++) {
    bb[i0][i1]+=op[i0][i1][i2][i3][i4][i5];
    }} }}}}

videocursor(0,BROW+8,0);
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
  printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);
videodot(199-(int)(100.0*bb[1][1]),BROW+(int)(100.0*b[1]),2);
}

/* end nxprob5t.c */
