
/* nxrijn5t.c						*/
/* program for lcau5t option 'd'			*/
/* display de Bruijn diagram - 1st, 2nd, & 3rd gen	*/
/* Harold V. McIntosh, 20 February 1988			*/
/* 21 February 1988 - add j0=i0; to Pass 4's		*/
/* 3 March 1988 - adapted from lca21.c to lca22.c	*/
/* 26 September 1989 - show evolution of periodic conf	*/
/* 2 October 1989 - cycl41.c incorporated into rijn41.c	*/
/* 20 November 1989 - removed loop from kwait		*/

/*  Copyright (C) 1987	*/
/*  Copyright (C) 1988	*/
/*  Harold V. McIntosh	*/
/*  Gerardo Cisneros S.	*/

#include <math.h>
#include "nxlcau5t.h"
#include "nxrijn5t.h"
#import  <dpsclient/psops.h>
#import  <appkit/graphics.h>

extern NXColor colst[KK];

extern char arule[DS], arr1[AL], arr2[AL], arr3[AL];
extern char ascrule[KK][KK][KK][KK];
extern int  binrule[KK][KK][KK][KK];
extern int  ix0, iy0;				/* origin for pen moves */

# define JX  0.50  /* x-center, de Bruijn diagram	*/
# define JY  0.50  /* y-center, de Bruijn diagram	*/
# define RI 0.425  /* inner radius, de Bruijn diagram	*/
# define RO 0.475  /* outer radius, de Bruijn diagram	*/
# define NW    24  /* pause after so many lines		*/

int  dd;		/* # points in de Bruijn ring	*/
int nc, nl;		/* column and line counters	*/
int mc, n_w;		/* column and line limits	*/
extern int    dbno;	/* node number in de Bruijn diagram 	*/

extern char           aarry[KK][KK][KK][KK];
extern char     barry[KK][KK][KK][KK][KK][KK][KK];

double dcntr[DDN][2];

/* edit the de Bruijn diagram		*/
/* g = number of generation (0==2nd)	*/
/* s = shift distance or constant val	*/
void edijn(g,s,p,r) int g, s, p, r; {

// printf("edijn %c, %d, %d, %d\n",g+'a',s,p,r);

   switch(g+'a') {
    case 'A': xpass1(s,r); break;
    case 'B': ypass1(s,r); break;

    case 'a': switch (s+'a'+1) {
//      case 'a': bpass1(9,p,r); break;
      case 'b': bpass1(10,p,r); break;
      case 'c': apass1(0,p,r); break;
      case 'd': apass1(1,p,r); break;
      case 'e': apass1(2,p,r); break;
      case 'f': apass1(3,p,r); break;
      case 'g': bpass1(11,p,r); break;
//      case 'h': bpass1(12,p,r); break;
      default: break;} break;

    case 'b': switch (s+'a') {
    case 'a': bpass1(0,p,r); break;
    case 'b': bpass1(1,p,r); break;
    case 'c': bpass1(2,p,r); break;
    case 'd': bpass1(3,p,r); break;
    case 'e': bpass1(4,p,r); break;
    case 'f': bpass1(5,p,r); break;
    case 'g': bpass1(6,p,r); break;
    case 'h': bpass1(7,p,r); break;
    case 'i': bpass1(8,p,r); break;
    default: break;} break;

    default: break;
    } /* end switch */
}

/* clear edijn screen and set up menu */
void clijn() {}

/* find the level of de Bruijn diagram appropriate to g, s  */
/* g = row, so g=0 means 2nd generation			    */
/* s = column, so 2s-5 is center for g even, s-3 for g odd  */
/* returns order of underlying de Bruijn diagram, 0=lowest  */
int shflev(g,s) int g, s; {int  i, k;
  k=g%2;
  if (k==0) {
    i=2*s-5;
    i=i<0?-i:i;
    i=(i-1)/2;
    if (i<(2+g/2)) i=g; else i=2*(i-1);
    }
  else {
    i=s-3;
    i=i<0?-i:i;
    if (i<=(g+1)/2) i=g; else i=2*i-1;
    }
// printf("shflev = %d\n",i);
  return i;
}

int permut(k,n) int k, n; {
  switch (n) {
    case 0: break;
    case 1: k=(k+1)%KK; break;
    default: break;}
  return k;
}

/* edit the Pass 5 (sample loops) constant displays */
void edshc(g,c) int g, c; {
  switch(g) {
    case 0: apass5(c); break;
    case 1: bpass5(c); break;
    default: break;}
}

/* edit the Pass 4 and Pass 5 shift displays */
/* pass5 fills the initial line of evolution */
/* shflev decides which connectivity matrix  */
void edshf(g,s,c) int g, s, c; {
    switch(shflev(g,s)) {
    case 0: apass5(c); break;
    case 1: bpass5(c); break;
//    case 2: cpass5(c); break;
//    case 3: dpass5(c); break;
//    case 4: epass5(c); break;
//    case 5: fpass5(c); break;
    default: break;
    } /* end switch */
}

/* edit the Pass 4 and Pass 5 shift displays */
/* pass6 draws the de Bruijn diagram 	     */
/* shflev decides which connectivity matrix  */
void edshg(g,s) int g, s; {
    switch(shflev(g,s)) {
    case 0: apass6(); break;
    case 1: bpass6(); break;
//    case 2: cpass6(); break;
//    case 3: dpass6(); break;
//    case 4: epass6(); break;
//    case 5: fpass6(); break;
    default: break;
    } /* end switch */
}

/* edit the Pass 4 and Pass 5 constant displays */
void edcst(s) int s; {
    switch(s) {
    case 0: apass6(); break;
    case 1: bpass6(); break;
//    case 2: cpass6(); break;
//    case 3: dpass6(); break;
//    case 4: epass6(); break;
//    case 5: fpass6(); break;
    default: break;
    } /* end switch */
}

/* increment ix */ 
void incix(ix,n) int *ix, n; {int i; 
  for (i=0; i<n; i++) {
    ix[i]++;
    if (ix[i]>=KK) ix[i]=0; else return;
    }
}

/* selected node in ix */ 
void nodix(ix,n) int *ix, n; {int i, j; 
  j=dbno;
  for (i=0; i<n; i++) {ix[n-i-1]=j%KK; j/=KK;}
}

/* random number in ix */ 
void ranix(ix,n) int *ix, n; {int i, j; 
  j=random();
  for (i=0; i<n; i++) {ix[i]=j%KK; j/=KK;}
}

/* clear ix */ 
void zerix(ix,n) int *ix, n; {int i; for (i=0; i<n; i++) ix[i]=0;}

/* find index of nearest center to x, y (de Bruijn) */
int nearest(x,y) double x, y; {int i, k; double ex, wy, r, p;
k=0; p=500000.0;
for (i=0; i<dd; i++) {
  ex=x-dcntr[i][0];
  wy=y-dcntr[i][1];
  r=ex*ex+wy*wy;
  if (r<p) {p=r; k=i;}
  }
return k;
}

/* - - - - - - - - d e   B r u i j n - - - - - - - - - - - - - - - - - - */

/* generate a de Bruijn diagram with m nodes, n links per node, color l */
void iijn(m,n,l) int m, n, l; {
int    i, j;
double ii, jj, nn, x, y, t, h;
nn=(double)(n);
t=6.28318/((double)(m));
h=0.5*t;
for (i=0; i<m; i++) {
  ii=(double)(i);
  for (j=0; j<n; j++) {
    jj=(double)(j);
    x=JX-RI*sin(ii*t+h);
    y=JY-RI*cos(ii*t+h);
    videoline(x,y,0);
    x=JX-RO*sin((nn*ii+jj)*t+h);
    y=JY-RO*cos((nn*ii+jj)*t+h);
    videoline(x,y,l);
    }
  }
}

/* generate a de Bruijn diagram with m nodes, n links per node, color l */
void ijn(m,n,l) int m, n, l; {
int    i, j;
double ii, jj, nn, x, y, t, h;
// printf("hello from ijn ... %3d, %3d, %3d\n",m,n,l);
nn=(double)(n);
t=6.28318/((double)(m));
h=0.5*t;
for (i=0; i<m; i++) {
  ii=(double)(i);
  for (j=0; j<n; j++) {
    jj=(double)(j);
    x=JX-RO*sin(ii*t+h);
    y=JY-RO*cos(ii*t+h);
    videoline(x,y,0);
    x=JX-RO*sin((nn*ii+jj)*t+h);
    y=JY-RO*cos((nn*ii+jj)*t+h);
    videoline(x,y,l);
    }
  }
}

/* insert a link into a de Bruijn diagram */
/* u - initial vertex     */
/* v - terminal vertex    */
/* l - color of link	  */

void lijnk(u,v,l) int u, v, l; {
char   s[10];
double x, y, ex, wy, eks, wye, dx, dy, r, w, t, sx, sy;

	if (l<KK) NXSetColor(colst[l]); else PSsetgray(NX_BLACK);
	r=10.3;
	w=3.33;
	ex=dcntr[u][0];
	wy=dcntr[u][1];
	x=dcntr[v][0];
	y=dcntr[v][1];
	dx=x-ex;
	dy=y-wy;
	t=atan2(dy,dx);
	sx=-w*sin(t);
	sy= w*cos(t);
	eks=x-r*cos(t)+sx;
	wye=y-r*sin(t)+sy;
	PSmoveto(ex+sx,wy+sy);
	PSlineto(eks,wye);
	PSlineto(eks-0.1*dx+0.05*dy,wye-0.1*dy-0.05*dx);
	PSlineto(eks-0.1*dx-0.05*dy,wye-0.1*dy+0.05*dx);
	PSlineto(eks,wye);

	PSmoveto(x+10.3,y);
	NXSetColor(colst[v]);
	PSarc(x, y, 10.3, 0.0, 360.0);
	PSstroke();

   PSsetgray(NX_BLACK); 

   ex=dcntr[u][0];
   wy=dcntr[u][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",u);
   PSshow(s);

   ex=dcntr[v][0];
   wy=dcntr[v][1];
   PSmoveto(ex-10.0,wy+5.0);
	sprintf(s,"%3d",v);
   PSshow(s);
   PSstroke();

}

/* generate a de Bruijn diagram showing ancestors of state k in color l */
void sijn(k,l) int k, l; {
int    i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
  if (k==ascrule[i0][i1][i2][i3]-'0') 
	lijnk(i0+KK*(i1+KK*i2),i1+KK*(i2+KK*i3),l);
}

/* generate a de Bruijn diagram showing the (1,k) links in color l */
void zijn(m,k,l) int m, k, l; {
int    i, i0, i1, i2, i3;
double th, x, y, t, h;
t=6.28318/((double)(m));
h=0.5*t;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
  i=ascrule[i0][i1][i2][i3]-'0';
  if ((k==0&&i==i0)||(k==1&&i==i1)||(k==2&&i==i2)||(k==3&&i==i3)) {
    th=((double)((i0*KK+i1)*KK+i2)*t+h);
    x=JX-RI*sin(th);
    y=JY-RI*cos(th);
    videoline(x,y,0);
    th=((double)((i1*KK+i2)*KK+i3)*t+h);
    x=JX-RO*sin(th);
    y=JY-RO*cos(th);
    videoline(x,y,l);
    }	     /* end if    */
  }}}}       /* end for's */
}	     /* end zijn  */

/* plot graph on video screen 		   */
/* move pen from present position to (x,y) */
/* raised if l=0; color l if l>0	   */
void videolyne(x,y,l) double x, y; int l; {
int k, ax, ay, dx, dy, di, dj, x1, y1;
/* if (x<0.0) return; if (x>1.0) return */
/* if (y<0.0) return; if (y>1.0) return */
x1=(int)(199.0*(1.0-y));y1=(int)(199.0*x);
dx=x1-ix0;
dy=y1-iy0;
ax=dx>=0?dx:-dx;
ay=dy>=0?dy:-dy;
di=dx>=0?1:-1; if (dx==0) di=0;
dj=dy>=0?1:-1; if (dy==0) dj=0;

if (l>0) {if (ax!=0 || ay!=0) {
  if (ax>=ay) {for (k=0; k<=ax; k++) videodot(ix0+di*k,60+iy0+(k*dy)/ax,l);}
	 else {for (k=0; k<=ay; k++) videodot(ix0+(k*dx)/ay,60+iy0+dj*k,l);}
  } }
ix0=x1;
iy0=y1;
}

/* plot graph on video screen 		   */
/* move pen from present position to (x,y) */
/* raised if l=0; color l if l>0	   */
void videoline(x,y,l) double x, y; int l; {
if (l==0) PSmoveto(400.0*x,400.0*y);
else PSlineto(400.0*x,400.0*y);
}

/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
/*									   */
/*		i0  i1  i2  i3  i4  i5  i6  i7  i8  i9 			   */
/*		      j0  j1  j2  j3  j4  j5  j6  			   */
/*		            k0  k1  k2  k3  				   */
/*				  l0					   */
/*									   */
/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

/* ONE5t.C */

/* Pass 1a analyzes all the configurations which fulfil (-1+l,1) */
void apass1(l,p,r) int l, p, r; {
int  ix[RR], m;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++) {
  switch (l) {
    case 0: m=ix[0]; break;
    case 1: m=ix[1]; break;
    case 2: m=ix[2]; break;
    case 3: m=ix[3]; break;
    default: break;}
  aarry[ix[0]][ix[1]][ix[2]][ix[3]]=
    binrule[ix[0]][ix[1]][ix[2]][ix[3]]==m?'Y':'N';
  }
if (r) {apass2i(); apass2o();}
}

/* Pass 1x analyzes all the configurations mapping into a constant */
void xpass1(l,r) int l; {
int  ix[RR];
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
  aarry[ix[0]][ix[1]][ix[2]][ix[3]]=
    (binrule[ix[0]][ix[1]][ix[2]][ix[3]]==l)?'Y':'N';
if (r) {apass2i(); apass2o();}
}

/* Pass 2i flags all links with an incoming arrow */
/* Pass 2o flags all links with an outgoing arrow */
/* Then pass 3 discards all unflagged links */
/* Passes 2 and 3 alternate until no change is observed */

void apass2i() {
int i0, i1, i2, i3, m;
do {
// printf(" Pass2i\015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
if ((aarry[i0][i1][i2][i3]&0x5F)=='Y')
  {for (m=0; m<KK; m++) aarry[i1][i2][i3][m]|=0x20;}
} } while (apass3()!=0); }

void apass2o() {
int i0, i1, i2, i3, m;
do {
// printf(" Pass2o\015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
if ((aarry[i0][i1][i2][i3]&0x5F)=='Y')
  {for (m=0; m<KK; m++) aarry[m][i0][i1][i2]|=0x20;}
} } while (apass3()!=0); }

/* Pass 3 - erase flags, mark survivors, detect changes */

int apass3() {
int i0, i1, i2, i3, l;
l=0;
// printf(" Pass3 \015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  switch (aarry[i0][i1][i2][i3]) {
    case 'y': aarry[i0][i1][i2][i3]='Y'; break;
    case 'Y': aarry[i0][i1][i2][i3]='N'; l=1; break;
    case 'n': case 'N': aarry[i0][i1][i2][i3]='N'; break;
    default: break; }
}
return l;
}

/* Pass 4 - print loops which remain */
void apass4() {
int i0, i1, i2, i3;
int j0, j1, j2, j3;
int k, l, m;
// printf(" pass4 \015");
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
j0=i0; j1=i1; j2=i2; j3=i3;
l=0; m=0;
do {
        if (aarry[0][j1][j2][j3]=='Y')
    {aarry[0][j1][j2][j3]='y';
    k=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (aarry[1][j1][j2][j3]=='Y')
    {aarry[1][j1][j2][j3]='y';
    k=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=j2; j2=j3; j3=k;} }}
  } while (l==0);
l=0; m=0;
do {
        if (aarry[j0][j1][j2][0]=='y')
   {aprf(j0,j1,j2,0);
   aarry[j0][j1][j2][0]='N';
   j0=j1; j1=j2; j2=0; m=1;}
  else {if (aarry[j0][j1][j2][1]=='y')
   {aprf(j0,j1,j2,1);
   aarry[j0][j1][j2][1]='N';
   j0=j1; j1=j2; j2=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (aarry[j0][j1][j2][0]=='Y')
   {aprf(j0,j1,j2,0);
   aarry[j0][j1][j2][0]='N';
   j0=j1; j1=j2; j2=0; m=1;}
  else {if (aarry[j0][j1][j2][1]=='Y')
   {aprf(j0,j1,j2,1);
   aarry[j0][j1][j2][1]='N';
   j0=j1; j1=j2; j2=1; m=1;}
  else l=1;}
  } while (l==0);
}}}}
}

/* print one link of the de Bruijn diagram */
void aprf(i0,i1,i2,i3) int i0, i1, i2, i3; {
printf("%1d",i0);
printf("%1d",i1);
printf("%1d",i2);
printf("-");
printf("%1d",i3);
printf("  ");
lijnk((i0*KK+i1)*KK+i2,(i1*KK+i2)*KK+i3,3);
}

/* Pass 5 - show evolution of sample loops */
void apass5(c) char c; {
int i0, i1, i2, i3;
int j1, j2, j3;
int hl;
int b[KK], irry[RR], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR); break;
    case 'r': ranix(irry,RR); break;
    case 'n': nodix(irry,RR-1);
	i0=irry[0]; i1=irry[1]; i2=irry[2];
	for (i=j=0; j<KK; j++) if (aarry[i0][i1][i2][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[3]=b[l];
	break;
    case 'i': incix(irry,RR); break;
    default:  zerix(irry,RR); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3];;
for (k=0; k<81; k++) {
  if (aarry[i0][i1][i2][i3]=='Y') break;
  incix(irry,RR);
  i0=irry[0]; i1=irry[1]; i2=irry[2]; i3=irry[3];;
  }
hl=AL/2;
arr1[hl-2]=i0;
arr1[hl-1]=i1;
arr1[hl]=i2;
arr1[hl+1]=i3;
j1=i1; j2=i2; j3=i3;
for (k=hl-3; k>=0; k--) {
  for (i=j=0; j<KK; j++) if (aarry[j][i0][i1][i2]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
  }
i1=j1; i2=j2; i3=j3;
for (k=hl+2; k<AL; k++) {
  for (i=j=0; j<KK; j++) if (aarry[i1][i2][i3][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i2; i2=i3; i3=arr1[k]=b[l];
  }
}

void apass6() {int i0, i1, i2, i3;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++){
  if(aarry[i0][i1][i2][i3]=='Y') lijnk((i0*KK+i1)*KK+i2,(i1*KK+i2)*KK+i3,3);
  }
}

/* TWO5t.C */

/* Pass 1 analyzes all sequences conforming to (2,-2+l)  */
void bpass1(l,p,r) int l, p, r; {
int ix[RR+3], jx[RR], kx;
int i, m, n;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++)
for (ix[6]=0; ix[6]<KK; ix[6]++) {
  for (i=0; i<RR; i++) jx[i]=binrule[ix[i]][ix[i+1]][ix[i+2]][ix[i+3]];
  kx=binrule[jx[0]][jx[1]][jx[2]][jx[3]];
switch (l) {
  case 0:  m=ix[0]; n=kx; break;
  case 1:  m=ix[1]; n=kx; break;
  case 2:  m=ix[2]; n=kx; break;
  case 3:  m=ix[3]; n=kx; break;
  case 4:  m=ix[4]; n=kx; break;
  case 5:  m=ix[5]; n=kx; break;
  case 6:  m=ix[6]; n=kx; break;
  case 9:  m=ix[0]; n=jx[2]; break;
  case 10: m=ix[6]; n=jx[2]; break;
  case 11: m=ix[1]; n=jx[2]; break;
  case 12: m=ix[5]; n=jx[2]; break;
  default: break;}
barry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]=m==n?'Y':'N';
}
if (r) {bpass2i(); bpass2o();}
}

/* Pass 1 analyzes all sequences conforming to (2,-2+l)  */
void ypass1(l,r) int l, r; {
int ix[RR+3], jx[RR], kx;
int i, m, n;
asctobin();
for (ix[0]=0; ix[0]<KK; ix[0]++)
for (ix[1]=0; ix[1]<KK; ix[1]++)
for (ix[2]=0; ix[2]<KK; ix[2]++)
for (ix[3]=0; ix[3]<KK; ix[3]++)
for (ix[4]=0; ix[4]<KK; ix[4]++)
for (ix[5]=0; ix[5]<KK; ix[5]++)
for (ix[6]=0; ix[6]<KK; ix[6]++) {
  for (i=0; i<RR; i++) jx[i]=binrule[ix[i]][ix[i+1]][ix[i+2]][ix[i+3]];
  kx=binrule[jx[0]][jx[1]][jx[2]][jx[3]];
  barry[ix[0]][ix[1]][ix[2]][ix[3]][ix[4]][ix[5]][ix[6]]=(kx==l?'Y':'N');
  }
if (r) {bpass2i(); bpass2o();}
}

/* Passs 2i flags links which have an incoming arrow */
void bpass2i() {
int i0, i1, i2, i3, i4, i5, i6 ,m;
do {
for (i0=0; i0<KK; i0++) 
for (i1=0; i1<KK; i1++) 
for (i2=0; i2<KK; i2++) 
for (i3=0; i3<KK; i3++) 
for (i4=0; i4<KK; i4++) 
for (i5=0; i5<KK; i5++) 
for (i6=0; i6<KK; i6++) {
if ((barry[i0][i1][i2][i3][i4][i5][i6]&0x5F)=='Y')
 {for (m=0; m<KK; m++) barry[i1][i2][i3][i4][i5][i6][m]|=0x20;}
}
} while (bpass3()!=0); }

/* Passs 2o flags links which have an outgoing arrow */
void bpass2o() {
int i0, i1, i2, i3, i4, i5, i6, m;
do {
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
if ((barry[i0][i1][i2][i3][i4][i5][i6]&0x5F)=='Y')
 {for (m=0; m<KK; m++) barry[m][i0][i1][i2][i3][i4][i5]|=0x20;}
}
} while (bpass3()!=0); }

/*  Pass 3 erases flags, marks survivors, counts channges */
int bpass3() {
int i0, i1, i2, i3, i4, i5, i6, l;
l=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
switch (barry[i0][i1][i2][i3][i4][i5][i6]) {
    case 'y': barry[i0][i1][i2][i3][i4][i5][i6]='Y'; break;
    case 'Y': barry[i0][i1][i2][i3][i4][i5][i6]='N'; l=1; break;
    case 'n': case 'N': barry[i0][i1][i2][i3][i4][i5][i6]='N'; break;
    default: break; }
}
return l;
}

/* Pass 4 prints loops which remain */
void bpass4() {
int i0, i1, i2, i3, i4, i5, i6;
int j0, j1, j2, j3, j4, j5, j6;
int k, l, m;
// printf(" pass4 \015");
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
j0=i0; j1=i1; j2=i2; j3=i3; j4=i4; j5=i5; j6=i6; l=0; m=0;
do {
        if (barry[0][j1][j2][j3][j4][j5][j6]=='Y')
    {barry[0][j1][j2][j3][j4][j5][j6]='y';
    k=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=0; m=1;}
  else {if (barry[1][j1][j2][j3][j4][j5][j6]=='Y')
    {barry[1][j1][j2][j3][j4][j5][j6]='y';
    k=j6; j6=j5; j5=j4; j4=j3; j3=j2; j2=j1; j1=1; m=1;}
  else {l=1; if (m==1) {j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=j6; j6=k;} }}
  } while (l==0);
l=0; 
m=0;
do {
        if (barry[j0][j1][j2][j3][j4][j5][0]=='y')
   {bprf(j0,j1,j2,j3,j4,j5,0);
   barry[j0][j1][j2][j3][j4][j5][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=0; m=1;}
  else {if (barry[j0][j1][j2][j3][j4][j5][1]=='y')
   {bprf(j0,j1,j2,j3,j4,j5,1);
   barry[j0][j1][j2][j3][j4][j5][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=1; m=1;}
  else {l=1;}}
  } while (l==0);
l=0;
do {
        if (barry[j0][j1][j2][j3][j4][j5][0]=='Y')
   {bprf(j0,j1,j2,j3,j4,j5,0);
   barry[j0][j1][j2][j3][j4][j5][0]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=0; m=1;}
  else {if (barry[j0][j1][j2][j3][j4][j5][1]=='Y')
   {bprf(j0,j1,j2,j3,j4,j5,1);
   barry[j0][j1][j2][j3][j4][j5][1]='N';
   j0=j1; j1=j2; j2=j3; j3=j4; j4=j5; j5=1; m=1;}
  else l=1; }
  } while (l==0);
}
}

/* print one of the links in a chain */
bprf(i0,i1,i2,i3,i4,i5,i6) int i0, i1, i2, i3, i4, i5, i6; {
int i, j;
printf("%1d",i0);
printf("%1d",i1);
printf("%1d",i2);
printf("%1d",i3);
printf("%1d",i4);
printf("%1d",i5);
printf("-");
printf("%1d",i6);
printf(" ");
i=((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5;
j=((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6;
lijnk(i,j,3);
}

/* Pass 5 - show evolution of sample loops */
void bpass5(c) char c; {
int i0, i1, i2, i3, i4, i5, i6;
int j1, j2, j3, j4, j5;
int hl;
int irry[RR+3], b[KK], i, j, k, l;
  switch (c) {
    case 'z': zerix(irry,RR+3); break;
    case 'r': ranix(irry,RR+3); break;
    case 'n': nodix(irry,RR+2);
	  i0=irry[0]; i1=irry[1]; i2=irry[2];
	  i3=irry[3]; i4=irry[4]; i5=irry[5];
	for (i=j=0; j<KK; j++) 
	  if (barry[i0][i1][i2][i3][i4][i5][j]=='Y') {b[i]=j; i++;}
	if (i==0) return;
	l=0; if (i>1) l=random()%i;
	irry[6]=b[l];
	break;
    case 'i': incix(irry,RR+3); break;
    default:  zerix(irry,RR+3); break;}
  i0=irry[0]; i1=irry[1]; i2=irry[2];
  i3=irry[3];
  i4=irry[4]; i5=irry[5]; i6=irry[6];
for (k=0; k<128; k++) {
  if (barry[i0][i1][i2][i3][i4][i5][i6]=='Y') break;
  incix(irry,RR+3);
  i0=irry[0]; i1=irry[1]; i2=irry[2];
  i3=irry[3]; i4=irry[4]; i5=irry[5]; i6=irry[6];
  }
hl=AL/2;
arr1[hl-3]=i0;
arr1[hl-2]=i1;
arr1[hl-1]=i2;
arr1[hl]=i3;
arr1[hl+1]=i4;
arr1[hl+2]=i5;
arr1[hl+3]=i6;
j1=i1; j2=i2; j3=i3; j4=i4; j5=i5;
for (k=hl-4; k>=0; k--) {
  for (i=j=0; j<KK; j++) 
	if (barry[j][i0][i1][i2][i3][i4][i5]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i5=i4; i4=i3; i3=i2; i2=i1; i1=i0; i0=arr1[k]=b[l];
  }
i1=j1; i2=j2; i3=j3; i4=j4; i5=j5;
for (k=hl+4; k<AL; k++) {
  for (i=j=0; j<KK; j++) 
	if (barry[i1][i2][i3][i4][i5][i6][j]=='Y') {b[i]=j; i++;}
  if (i==0) return;
  l=0; if (i>1) l=random()%i;
  i1=i2; i2=i3; i3=i4; i4=i5; i5=i6; i6=arr1[k]=b[l];
  }
}

void bpass6() {int i, j, i0, i1, i2, i3, i4, i5, i6;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  i=((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5;
  j=((((i1*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6;
  if(barry[i0][i1][i2][i3][i4][i5][i6]=='Y') lijnk(i,j,3);
  }
}


/* TWO5T.C */

/* end nxrijn5t.c */
